/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Aug 9, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.client.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import xmlbeans.org.oasis.saml2.assertion.AssertionDocument;

import eu.unicore.security.etd.TrustDelegation;
import eu.unicore.util.Log;
import eu.unicore.util.configuration.ConfigurationException;
import eu.unicore.util.configuration.DocumentationReferenceMeta;
import eu.unicore.util.configuration.DocumentationReferencePrefix;
import eu.unicore.util.configuration.FilePropertiesHelper;
import eu.unicore.util.configuration.PropertiesHelper;
import eu.unicore.util.configuration.PropertyMD;
import eu.unicore.util.configuration.PropertyMD.DocumentationCategory;
import eu.unicore.util.httpclient.ClientProperties;


/**
 * @author K. Benedyczak
 */
public class UVOSClientProperties extends ClientProperties
{
	@DocumentationReferencePrefix
	public static final String PREFIX = "uvos.";
	private static final Logger log = Log.getLogger(Log.CONFIGURATION, UVOSClientProperties.class);
	
	public static final String PROP_SERVER_HOST = "host";
	public static final String PROP_SERVER_PORT = "port";
	public static final String PROP_SERVER_PROTOCOL = "protocol";
	public static final String PROP_SERVICE_PATH = "servicePath";
	public static final String PROP_ETD = "etdFile";

	@DocumentationReferenceMeta
	public final static Map<String, PropertyMD> DEFAULTS = new HashMap<String, PropertyMD>();
	static 
	{
		DocumentationCategory main = new DocumentationCategory("Main", "a");
		DocumentationCategory extra = new DocumentationCategory("Additional", "b");
		
		DEFAULTS.put(PROP_SERVER_HOST, new PropertyMD("localhost").setCategory(main).
				setDescription("UVOS server host"));
		DEFAULTS.put(PROP_SERVER_PORT, new PropertyMD("2443").setBounds(1, 65535).setCategory(main).
				setDescription("UVOS server port"));
		DEFAULTS.put(PROP_SERVER_PROTOCOL, new PropertyMD("https").setCategory(main).
				setDescription("Protocol to use. 'http' or (in the most cases) 'https'."));
		DEFAULTS.put(PROP_SERVICE_PATH, new PropertyMD("").setCategory(extra).
				setDescription("Service path. For UVOS it should be left empty, " +
						"for other implementations its deployment path should be given."));
		DEFAULTS.put(PROP_ETD, new PropertyMD().setPath().setCategory(extra).
				setDescription("Optional path of a file with a trust delegation which should be used."));
		
		for (String key: ClientProperties.META.keySet())
			DEFAULTS.put(key, ClientProperties.META.get(key).clone().setCategory(extra));
	}
	
	protected String host;
	protected int port;
	protected String protocol;
	protected String servicePath;
	protected File etdPath;
	
	public UVOSClientProperties(String file) throws ConfigurationException, IOException
	{
		this(FilePropertiesHelper.load(file));
	}
	
	public UVOSClientProperties(Properties source) throws ConfigurationException, IOException
	{
		super(source);
		PropertiesHelper properties = new PropertiesHelper(PREFIX, source, 
				DEFAULTS, log);
		host = properties.getValue(PROP_SERVER_HOST);
		port = properties.getIntValue(PROP_SERVER_PORT);
		protocol = properties.getValue(PROP_SERVER_PROTOCOL);
		servicePath = properties.getValue(PROP_SERVICE_PATH);
		setEtdPath(properties.getFileValue(PROP_ETD, false));
	}
	
	/**
	 * only for cloning
	 */
	protected UVOSClientProperties()
	{
	}
	
	public int getServerPort()
	{
		return port;
	}

	public String getServerHost()
	{
		return host;
	}

	public String getProtocol()
	{
		return protocol;
	}
	
	public String getServicePath()
	{
		return protocol;
	}

	public void setHost(String host)
	{
		this.host = host;
	}

	public void setPort(int port)
	{
		this.port = port;
	}

	public void setProtocol(String protocol)
	{
		this.protocol = protocol;
	}

	public void setServicePath(String servicePath)
	{
		this.servicePath = servicePath;
	}
	
	public File getEtdPath()
	{
		return etdPath;
	}

	public void setEtdPath(File etdPath) throws IOException
	{
		this.etdPath = etdPath;
		if (etdPath == null)
			return;
		List<TrustDelegation> tdTokens = getTDTokens(etdPath);
		getETDSettings().setTrustDelegationTokens(tdTokens);
	}

	protected List<TrustDelegation> getTDTokens(File tdFile) throws IOException
	{
		try{
			AssertionDocument td=AssertionDocument.Factory.parse(tdFile);
			List<TrustDelegation>res=new ArrayList<TrustDelegation>();
			res.add(new TrustDelegation(td));
			return res;
		}catch(Exception xe){
			IOException e= new IOException("Attempted to create trust delegation token from invalid XML.");
			e.initCause(xe);
			throw e;
		}
	}

	
	@Override
	public UVOSClientProperties clone()
	{
		UVOSClientProperties ret = new UVOSClientProperties();
		cloneTo(ret);
		ret.setServicePath(servicePath);
		ret.setProtocol(protocol);
		ret.setHost(host);
		ret.setPort(port);
		ret.etdPath = etdPath;
		ret.getETDSettings().setTrustDelegationTokens(getETDSettings().getTrustDelegationTokens());
		return ret;
	}
}
