/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;

import eu.emi.security.authn.x509.X509CertChainValidator;
import eu.emi.security.authn.x509.X509Credential;
import eu.emi.security.authn.x509.impl.X500NameUtils;
import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.proto.AbstractRequest;
import eu.unicore.samly2.proto.AbstractStatusResponse;
import eu.unicore.security.dsig.DSigException;

/**
 * This class provides a base for Client implementations.
 * It merely provides code to handle signatures.
 *
 * @author K. Benedyczak
 */
public abstract class AbstractSAMLBase
{
	protected boolean requireSignedResp, requireSignedAssertion, doSignRequest;
	protected PublicKey publicKey;

	private transient PrivateKey privKey;
	protected X509Certificate[] certificateC;
	private transient X509CertChainValidator validator;

	/**
	 * 
	 * @param secCfg credential from this object is used for creating signatures. 
	 * Validator is used only during verification when the verification key
	 * was not set manually.
	 * @throws SecuritySetupException
	 * @throws MalformedURLException
	 */
	protected AbstractSAMLBase() 
		throws SecuritySetupException, MalformedURLException
	{
		requireSignedAssertion = false;
		requireSignedResp = false;
		doSignRequest = false;
	}

	/**
	 * Set signing policy, i.e. what is required and checked in received SAML assertions.
	 * This method variant sets a fixed verification key, which must be trusted. Signature of 
	 * a SAML response or assertion will be accepted only if is matching to this key, but this key 
	 * itself is not anyhow verified. 
	 * @param requireSignedReqResp We require that the response must be signed. Usually this is not needed.
	 * This argument also controls whether queries are signed.
	 * @param requireSignedAssertion We require that the assertion in response should be signed.
	 * @param credential Credential used for creating signatures. May be null (is ignored) if requireSignedReqResp
	 * is false
	 * @param publicKey Public key which should be used for signature verification. May be null only if both 
	 * require arguments are false 
	 */
	public void setDSigPolicy(boolean requireSignedReqResp, boolean requireSignedAssertion,
			X509Credential credential, PublicKey publicKey)
	{
		if (publicKey == null && (requireSignedAssertion || requireSignedReqResp))
			throw new IllegalArgumentException("Public key can not be null " +
					"if signature checking is required.");
		setDSigPolicyCommon(requireSignedReqResp, requireSignedAssertion, credential);
		this.publicKey = publicKey;
		validator = null;
	}

	/**
	 * Set signing policy, i.e. what is required and checked in received SAML assertions.
	 * This method variant sets a fixed verification key, which must be trusted. Signature of 
	 * a SAML response or assertion will be accepted only if is matching to this key, but this key 
	 * itself is not anyhow verified. 
	 * @param requireSignedReqResp We require that the response must be signed. Usually this is not needed.
	 * This argument also controls whether queries are signed.
	 * @param requireSignedAssertion We require that the assertion in response should be signed.
	 * @param credential Credential used for creating signatures. May be null (is ignored) if requireSignedReqResp
	 * is false
	 * @param validator object used to validate the issuer certificate of a signature. Remember that usually this
	 * should be a different validator then the (much broader) one used for TLS connections. Typically it
	 * should contain only the trusted SAML authority certificates in its trust store.
	 */
	public void setDSigPolicy(boolean requireSignedReqResp, boolean requireSignedAssertion,
			X509Credential credential, X509CertChainValidator validator)
	{
		if (validator == null && (requireSignedAssertion || requireSignedReqResp))
			throw new IllegalArgumentException("Validator can not be null " +
					"if signature checking is required.");
		setDSigPolicyCommon(requireSignedReqResp, requireSignedAssertion, credential);
		this.publicKey = null;
		this.validator = validator;
	}

	/*-********************************************************
	 * utility methods 
	 *-********************************************************/

	protected void setDSigPolicyCommon(boolean requireSignedReqResp, boolean requireSignedAssertion,
			X509Credential credential)
	{
		if (credential == null && requireSignedReqResp)
			throw new IllegalArgumentException("Credential can not be null " +
					"if request signing is required.");
		
		this.requireSignedAssertion = requireSignedAssertion;
		this.requireSignedResp = requireSignedReqResp;
		this.doSignRequest = requireSignedReqResp;
		if (credential != null)
		{
			this.certificateC = credential.getCertificateChain();
			this.privKey = credential.getKey();
		} else
		{
			this.certificateC = null;
			this.privKey = null;
		}
	}
	
	protected void signIfNeeded(AbstractRequest req) throws InvalidSignatureException
	{
		if (doSignRequest)
		{
			try
			{
				req.sign(privKey, certificateC);
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Can't sign request: " 
						+ e);
			}
		}	
	}
	
	protected void checkResponseSignature(AbstractStatusResponse resp) 
		throws InvalidSignatureException
	{
		if (!requireSignedResp)
			return;
		if (publicKey != null)
		{
			try
			{
				if (!resp.isCorrectlySigned(publicKey))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException(
					"Problem when checking " +
					"response signature with given pub key: " + e);
			}
		} else
		{
			X509Certificate[] issuersCC = resp.getIssuerFromSignature();
			if (issuersCC == null || issuersCC.length == 0)
				throw new InvalidSignatureException(
						"Response doesn't contain " +
						"issuer's certificate and public key" +
				" for verification wasn't specified");
			try
			{
				checkIfTrusted(issuersCC, "Response");
				if (!resp.isCorrectlySigned(issuersCC[0].getPublicKey()))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException(
					"Problem when checking " +
					"response signature with extracted pub key: " + e);
			}
		}
	}

	protected void checkAssertionSignature(AbstractStatusResponse resp, 
			Assertion assertion) throws InvalidSignatureException
	{
		if (!requireSignedAssertion)
			return;
		if (publicKey != null)
		{
			try
			{
				if (!assertion.isCorrectlySigned(publicKey))
					throw new InvalidSignatureException(
					"Assertion is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Problem when checking " +
						"response assertion signature with given " +
						"pub key: " + e);
			}
		} else
		{
			X509Certificate[] issuersCC = assertion.
				getIssuerFromSignature();
			if (issuersCC == null || issuersCC.length == 0)
			{
				issuersCC = resp.getIssuerFromSignature();
				if (issuersCC == null || issuersCC.length == 0)
					throw new InvalidSignatureException(
						"Assertion doesn't contain " +
						"issuer's certificate and public key" +
						" for verification wasn't specified");
			}
			try
			{
				checkIfTrusted(issuersCC, "Assertion");
				if (!assertion.isCorrectlySigned(issuersCC[0].getPublicKey()))
					throw new InvalidSignatureException(
					"Response is incorrectly signed");
			} catch (DSigException e)
			{
				throw new InvalidSignatureException("Problem when checking " +
						"response assertion signature with extracted " +
						"pub key: " + e);
			}
		}
	}
	
	
	private void checkIfTrusted(X509Certificate[] issuersCC, String msg) throws InvalidSignatureException
	{
		X509Certificate[] trustedIssuers = validator.getTrustedIssuers();
		boolean found = false;
		for (X509Certificate trusted: trustedIssuers)
		{
			if (trusted.equals(issuersCC[0]))
			{
				found = true;
				break;
			}
		}
		if (!found)
			throw new InvalidSignatureException(
				msg + " is signed by an issuer is not among trusted issuers" +
				" and verification key was not set automaticvally. Untrusted issuer is " 
				+ X500NameUtils.getReadableForm(issuersCC[0].getSubjectX500Principal()));
	}
}
