/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.security.PublicKey;

import eu.emi.security.authn.x509.X509CertChainValidator;
import eu.emi.security.authn.x509.X509Credential;
import eu.unicore.samly2.assertion.Assertion;
import eu.unicore.samly2.exceptions.SAMLParseException;
import eu.unicore.samly2.proto.AssertionResponse;
import eu.unicore.util.httpclient.IClientConfiguration;


import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.SAMLAuthnInterface;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UnsupportedSAMLException;
import xmlbeans.org.oasis.saml2.protocol.AuthnRequestDocument;
import xmlbeans.org.oasis.saml2.protocol.ResponseDocument;

/**
 * Implementation of utility methods for SAML authentication protocol.
 * See {@link SAMLVOAuthnClientInterface}.
 * 
 * Signatures note: As suggested by spec this implementation:
 * <ul>
 * <li>Always requires signed assertion in response.
 * <li>Always signing the request.
 * </ul> 
 * @author K. Benedyczak
 */
public class SAMLVOAuthnClientSOAP extends SAMLVOClient
{
	private SAMLAuthnInterface proxy;
	private SAMLVOAuthnClientPOST util;
	
	/**
	 * Use this constructor if you want to use SAML SOAP binding.
	 * @param address server address to use.  
	 * @param secProv security configuration (used to setup TLS and to sign messages)
	 * @param issuer URI which will be used as SAML request issuer (of SAML type 'entity').
	 * @param validator used to check assertion signature issuer
	 * @throws SecuritySetupException
	 * @throws MalformedURLException
	 */
	public SAMLVOAuthnClientSOAP(String address, IClientConfiguration secProv,
			URI issuer, X509CertChainValidator validator)
		throws SecuritySetupException, MalformedURLException
	{
		super(address, secProv, issuer);
		setDSigPolicy(true, true, secProv.getCredential(), validator);
		try
		{
			if (proxy == null)
				proxy = fact.getSAMLAuthnClient(address);
		} catch (MalformedURLException e){}
		util = new SAMLVOAuthnClientPOST(secProv, issuer, issuer, validator);
	}

	@Override
	public Object getProxy()
	{
		return proxy;
	}

	@Override
	public void setDSigPolicy(boolean requireSignedReqResp, boolean requireSignedAssertion,
			X509Credential credential, PublicKey publicKey)
	{
		if (!requireSignedAssertion)
			throw new IllegalArgumentException("For SAML authentication, the signatures of " +
					"response assertions must be required");
		super.setDSigPolicy(requireSignedReqResp, requireSignedAssertion, credential, publicKey);
		requireSignedResp = false;
	}

	@Override
	public void setDSigPolicy(boolean requireSignedReqResp, boolean requireSignedAssertion,
			X509Credential credential, X509CertChainValidator validator)
	{
		if (!requireSignedAssertion)
			throw new IllegalArgumentException("For SAML authentication, the signatures of " +
					"response assertions must be required");
		super.setDSigPolicy(requireSignedReqResp, requireSignedAssertion, credential, validator);
		requireSignedResp = false;
	}

	/**
	 * Performs SOAP request.
	 * @param requestedFormat requested SAML name format to be returned.
	 * @return
	 * @throws SAMLParseException
	 * @throws UnsupportedSAMLException
	 * @throws InvalidSignatureException
	 * @throws InvocationException
	 * @throws SAMLErrorResponseException
	 */
	public Assertion authenticate(String requestedFormat) throws SAMLParseException, 
		UnsupportedSAMLException, InvalidSignatureException, InvocationException,
		SAMLErrorResponseException
	{
		AuthnRequestDocument reqDoc = util.createRequest(requestedFormat).getDoc();
		ResponseDocument xmlRespDoc;
		try
		{
			xmlRespDoc = proxy.authnRequest(reqDoc);
			lastResponse = xmlRespDoc;
		} catch (UVOSFault e)
		{
			throw new InvocationException(e);
		}
		
		AssertionResponse resp = new AssertionResponse(xmlRespDoc);
		resp.parse();
		
		if (!resp.isStatusOK())
			throw new SAMLErrorResponseException(resp.getErrorMessage(),
					resp.getErrorStatus(), resp.getSubErrorStatus());

		checkResponseSignature(resp);

		Assertion[] assertions;
		try
		{
			assertions = resp.getAssertions();
		} catch (Exception e)
		{
			throw new SAMLParseException(e.toString());
		}
		
		if (assertions.length == 0)
			return null;
		if (assertions.length > 1)
			throw new UnsupportedSAMLException(
				"More than one assertion was returned. It is OK," +
				"however this implementation supports only " +
				"responses with a single assertion.");
		Assertion assertion = assertions[0];

		checkAssertionSignature(resp, assertion);

		return assertion;
	}
}
